var console = {
  log: function() {
    var i
    for ( i = 0; i < arguments.length; i++ )
      Ti.API.info( arguments[ i ])
  },

  dir: function() {
    console.log.apply(
      this,
      Array.prototype.map.call( arguments, function( arg ) {
        return JSON.stringify( arg )
      })
    )
  }
}


var tiQuery = (function() {

  var filter = function( items, iterator, count ) {
    var memo = [], i = 0
    if ( typeof count === 'undefined' ) count = items.length

    while ( i < items.length && memo.length < count ) {
      if ( iterator.call( items[i], items[i], i, items ))
        memo.push( items[i] )
      i++
    }

    return memo
  }

  var Tizzle = {}
  Tizzle.selectors = [
    {
      regex: /#([\w-_]+)/,
      test: function( match ) { return this.id === match[1] },
      abortOnFirst: true
    }, {
      regex: /\.([\w-_]+)/,
      test: function( match ) { return this.className === match[1] }
    }, {
      regex: /^\*$/,
      test: function( match ) { return true }
    }
  ]


  Tizzle.filter = function( selectorString, elements ) {
    var filteredElements = []

    if ( typeof elements !== 'undefined' ) {
      selectorString.split( /,\s+/ ).forEach( function( subSelectorString ) {
        var selector = filter( Tizzle.selectors, function( selector ) {
          return subSelectorString.match( selector.regex )
        }, 1 )[ 0 ]

        if ( typeof selector === 'undefined' )
          return console.log("TRIED TO USE UNKNOWN SELECTOR " + subSelectorString)

        var match = subSelectorString.match( selector.regex )

        filteredElements = filteredElements.concat(
          filter( elements, function( element ) {
            return selector.test.call( element, match )
          }, selector.abortOnFirst ? 1 : undefined )
        )
      })
    }

    return filteredElements
  }

  Tizzle.find = function( selectorString, elements ) {
    if (! elements) return []

    var matches = Tizzle.filter( selectorString, elements )

    elements.forEach(function( child ) {
      if ( child ) {
        //console.log('tizzle searching: ' + child.id + ':' + child.kids)
        if (child.kids && child.kids.length ) {
          //console.log('Tizzle.filter looking up kids: ' + selectorString)
          matches.push.apply( matches, Tizzle.find( selectorString, child.kids ))
        } else if (child.children && child.children.length) {
          matches.push.apply( matches, Tizzle.find( selectorString, child.children ))
        }
      }
    })

    return matches
  }

  var isArray = function( thing ) {
    return Object.prototype.toString.apply( thing ) === '[object Array]'
  }

  var flatten = function( arr ) {
    var newArr = []

    arr.forEach( function( item ) {
      if ( isArray( item )) newArr.push.apply( newArr, flatten( item ))
      else newArr.push( item )
    })

    return newArr
  }

  var makeArray = function( thing, results ) {
    var returnValue = results || []

    if ( Array.isArray( thing )) {

    } else {
      returnValue.push( thing )
      return returnValue
    }

    return Array.isArray( thing )
      ? flatten( thing )
      : [ thing ]
  }

  var tiQuery = function( selector, knownTitaniumObject ) {
    //console.log('tiQuery:' + selector + ':' + knownTitaniumObject)
    return typeof selector === 'object' && selector && selector.__tiQuery
      ? selector
      : new tiQuery.fn.init( selector, knownTitaniumObject )
  }

  tiQuery.tizzle = Tizzle

  tiQuery.cache = {}

  tiQuery.cache.ids = {}

  tiQuery.cache.clear = function( id ) {
    tiQuery.cache.ids = {}
  }

  tiQuery.cache.add = function( thing ) {
    tiQuery.cache.ids[ '#' + thing.id ] = thing
  }

  tiQuery.cache.find = function( selector ) {
    var object = tiQuery.cache.ids[ selector ]

    if ( object && '#' + object.id === selector )
      return tiQuery( object )
  }

  tiQuery.cache.rename = function( idBefore, idNow ) {
    tiQuery.cache.ids[ '#' + idNow ] = tiQuery.cache.ids[ '#' + idBefore ]
    tiQuery.cache.remove( idBefore )
  }

  tiQuery.cache.remove = function( id ) {
    delete tiQuery.cache.ids[ '#' + id ]
  }



  tiQuery.bind = function( fn, toBeThis ) {
    return function() { return fn.apply( toBeThis, arguments ) }
  }

  tiQuery.isWrapped = function( obj ) {
    return !!(obj && obj.__tiQuery)
  }

  tiQuery.isTitaniumObject = function( object ) {
    return object && object.toImage
  }

  tiQuery.clamp = function( val, min, max ) {
    return Math.min( max, Math.max( min, val ))
  }

  tiQuery.extract = function( object, key, defaultValue ) {
    var value = object[ key ]
    delete object[ key ]
    return typeof value === 'undefined'
      ? defaultValue
      : value
  }

  tiQuery.initialCaps = function( string ) {
    return string.replace( /(?:^|\s+)[a-z]/g, function( match ) {
      return match.toUpperCase()
    })
  }

  tiQuery.camelToSnake = function( string ) {
    return string.replace( /[A-Z]+/g, function( match ) {
      return '_' + match.toLowerCase()
    }).replace( /^_/, '' )
  }

  tiQuery.snakeToCamel = function( string, initialCaps ) {
    var regex = initialCaps ? /(?:^|_)([a-z])/g : /_([a-z])/g
    return string.replace( regex, function( match, letter ) {
      return letter.toUpperCase()
    })
  }

  tiQuery.cast = function( input ) {
    if ( typeof input !== 'string' ) return input
    if ( input.match( /^[0-9]+$/ )) return parseInt( input, 10 )
    if ( input.match( /^0x[0-9A-F]+$/i )) return parseInt( input, 16 )
    if ( input.match( /^[0-9]+\.[0-9]*$/ )) return parseFloat( input )
    if ( input.match( /^true$/i )) return true
    if ( input.match( /^false$/i )) return false
    return input
  }

  tiQuery.fn = tiQuery.prototype = {
    constructor: tiQuery,
    __tiQuery: true
  }

  tiQuery.fn.init = function( selector, knownTitaniumObject ) {
    //console.log('tiQuery.fn.init')
    this.length = 0
    if ( knownTitaniumObject || tiQuery.isTitaniumObject( selector ) ) {
      //console.log('titanium object')
      this.context = this[0] = selector
      this.length = 1
      return this
    } else if ( typeof selector === 'function' ) {
      return tiQuery.root.on( 'transitionfinished', selector )
    } else if ( Array.isArray( selector )) {
      for (var i = 0 ; i < selector.length ; i++ )
        this[ i ] = selector[ i ]
      this.length = selector.length
      return this
    } else if ( selector && typeof selector === 'object' ) {
      return tiQuery.build( selector )
    } else if ( typeof selector === 'string' ) {
      var found = tiQuery.cache.find( selector )
      if (!found) {
        console.log('Cache miss for ' + selector + ', falling back to tizzle find')
        found = tiQuery.root.find( selector )
      } else {
        //console.log('Found ' + selector + 'in cache')
      }
      if (!found.length) {
        throw "Didn't find items with selector: " + selector
      }
      return found
    }
  }

  tiQuery.fn.init.prototype = tiQuery.fn


  tiQuery.fn.each = function( func ) {
    var index, indexElement
    for ( index = 0; index < this.length; index++ ) {
      indexElement = this[ index ]
      if ( indexElement.__tiQuery ) indexElement.each( func )
      else func.call( indexElement )
    }
    return this
  }

  tiQuery.getters = {
    border: function() {
      return (this.borderWidth || 0) + " " +
        (this.borderColor || "black")
    },

    background: function() {
      return this.backgroundColor
    },

    classes: function() {
      return (this.className || '').split(/\s+/)
    },

    right: function() {
      return this.left + this.width
    },

    bottom: function() {
      return this.top + this.height
    },

    center: function() {
      return {x: this.left + this.width/2, y: this.top + this.height/2}
    }
  }

  var postBuild = function( fn ) {
    fn.postBuild = true
    return fn
  }

  tiQuery.setters = {
    id: function( id ) {
      if ( this.id ) tiQuery.cache.rename( this.id, id )
      this.id = id
    },

    border: function( border ) {
      var parts = border.split( /\s+/ )
      this.borderWidth = parts[0].replace(/px$/,'') || 0
      this.borderColor = parts[1] || 'black'
    },

    systemButton: function( buttonName ) {
      this.systemButton = typeof buttonName === 'string'
        ? Titanium.UI.iPhone.SystemButton[ buttonName.toUpperCase() ]
        : buttonName
    },

    keyboardType: function( keyboardType ) {
      this.keyboardType = typeof keyboardType === 'string'
        ? Titanium.UI[ 'KEYBOARD_' + keyboardType.toUpperCase() ]
        : keyboardType
    },

    pickerType: function( pickerType ) {
      this.type = typeof pickerType === 'string'
        ? Titanium.UI[ 'PICKER_TYPE_' + pickerType.toUpperCase() ]
        : pickerType
    },

    curve: function( curve ) {
      this.curve = typeof curve === 'string'
        ? Titanium.UI[ 'ANIMATION_CURVE_' + curve.toUpperCase() ]
        : curve
    },

    buttonStyle: function( buttonStyle ) {
      this.style = Titanium.UI.iPhone.SystemButtonStyle[ buttonStyle.toUpperCase() ]
    },

    indicatorStyle: function( style ) {
      this.style = Titanium.UI.iPhone.ActivityIndicatorStyle[ style.toUpperCase() ]
    },

    background: function( background ) {
      this.backgroundColor = background
    },

    classes: function( classes ) {
      this.className = typeof classes === 'string'
        ? classes
        : classes.join( ' ' )
    },

    rotate: function( angle ) {
      this.transform = Titanium.UI.create2DMatrix().rotate( angle )
      this.rotate = angle
    },

    scale: function( scale ) {
      this.transform = Titanium.UI.create2DMatrix().scale( scale )
      this.scale = scale
    },

    selectionStyle: function( styleString ) {
      this.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle[ styleString.toUpperCase() ]
    },

    right: function( right ) {
      this.left = right - this.width
    },

    bottom: function( bottom ) {
      this.top = bottom - this.height
    },

    center: function( point ) {
      if( typeof point.x !== 'undefined' ){
        if( this.object ) this.width = this.object.width
        this.left = point.x - this.width/2
      }
      if( typeof point.y !== 'undefined' ){
        if( this.object ) this.height = this.object.height
        this.top  = point.y - this.height/2
      }
    },

    children: postBuild( function( children ) {
      var parent = tiQuery(this)
      children.forEach( function ( childAttrs ) {
        parent.add( tiQuery( childAttrs ))
      })
    }),

    template: postBuild( function( template ) {
      $(this).addTemplate( template )
    })
  }


  tiQuery.fn.toggle = function( value ) {
    return this.each( function() {
      ( typeof value === 'undefined' &&
        $(this).attr( 'visible' )) || ! value
        ? this.hide()
        : this.show()
    })
  }


  tiQuery.fn.appendRow = function( row ) {
    return this.each( function() {
      var parent = this
      tiQuery( row ).each( function() {
        parent.appendRow( this )
      })
    })
  }

  tiQuery.fn.scrollToTop = function() {
    return this.each( function() {
      if ( typeof this.scrollToTop === 'function' )
        this.scrollToTop( 0 )
      else if ( typeof this.scrollTo === 'function' )
        this.scrollTo( 0, 0 )
    })
  }

  tiQuery.fn.animate = function( attributes, callback ) {
    return this.each( function() {
      var tiObject = tiQuery(this)

      var animation = {object: this}

      Object.keys( attributes ).forEach( function( attrName ) {
        var attrValue = attributes[ attrName ]
        if ( tiQuery.setters[ attrName ]) {
          tiQuery.setters[ attrName ].call( animation, attrValue )
        } else {
          animation[ attrName ] = attrValue
        }
      })

      if (typeof animation.duration === 'undefined') animation.duration = 100

      delete animation.object

      this.animate( animation, function() {
        tiObject.attr( attributes )
        if ( typeof callback === 'function' )
          callback.apply( tiObject, arguments )
        tiObject = callback = undefined
      })
    })
  }

  tiQuery.builders = {
    image: Ti.UI.createImageView,
    tableRow: Ti.UI.createTableViewRow,
    textfield: Ti.UI.createTextField
  }

  tiQuery.builder = function( builder ) {
    var buildFunc = tiQuery.builders[ builder ] ||
      Ti.UI[ 'create' + builder[0].toUpperCase() + builder.slice( 1 ) ]

    return buildFunc
  }

  tiQuery.build = function( attributes ) {
    var type    = attributes.type || 'view'
    ,   builder = tiQuery.builder( type )
    ,   processedAttributes = {}

    var postAttributes = {}
    Object.keys( attributes ).forEach( function ( attrName ) {
      if ( tiQuery.setters[ attrName ] ) {
        if ( tiQuery.setters[ attrName ].postBuild ) {
          postAttributes[ attrName ] = attributes[ attrName ]
        } else {
          tiQuery.setters[ attrName ].call( processedAttributes,
                                            attributes[ attrName ] )
        }
      } else {
        processedAttributes[ attrName ] = attributes[ attrName ]
      }
    })

    var view = builder( processedAttributes )

    Object.keys( postAttributes ).forEach( function ( attrName ) {
      tiQuery.setters[ attrName ].call( view,
                                        attributes[ attrName ] )
    })

    return tiQuery( view, true )
  }

  tiQuery.fn.text = function( text ) {
    return this.attr( 'text', text )
  }

  tiQuery.fn.val = function( value ) {
    return this.attr( 'value', value )
  }

  tiQuery.fn.size = function() {
    return this.length
  }

  tiQuery.fn.add = function( thing ) {
    return this.each(function() {
      var parent = this

      tiQuery( thing ).each( function() {
        //console.log('adding to parent: ' + parent.id + ':'+ this.id)
        var kids = parent.kids
        if (!kids)
          kids = [ this ]
        else
          kids.push( this )
        parent.kids = kids
        //console.log('  typeof kids:' + typeof parent.kids)
        //console.log('  kids: ' + JSON.stringify(parent.kids) )
        parent.add( this )
        tiQuery.cache.add( this )
      })
    })
  }

  tiQuery.fn.remove = function( thing ) {
    return this.each(function() {
      var parent = this

      tiQuery( thing ).each( function() {
        if (!parent.kids)
        delete parent.kids[thing]
        parent.remove( this )
        tiQuery.cache.remove( this.id )
      })
    })
  }
  tiQuery.fn.addTemplate = function( thing ) {
    var parent = this
    ,   template
    if (typeof thing === 'string') {
      template = tiQuery.loadJSON( thing )
    } else if ( isArray( thing )) {
      template = thing
    } else {
      throw 'Invalid object passed to addTemplate ' + thing
    }

    template.forEach( function( item ) {
      parent.add( item )
    })

    return parent
  }

  tiQuery.fn.addTo = function( thing ) {
    tiQuery( thing ).add( this )
    return this
  }

  ;'hide show open close'.split(' ')
    .forEach(function( actionName ) {
      tiQuery.fn[ actionName ] = function() {
        return this.each( function() {
          this[ actionName ]()
        })
      }
    })

  tiQuery.fn.pushStack = function() {
    var object = this.constructor()

    object.previousObject = this
    object.context = this.context

    return object
  }

  tiQuery.fn.end = function() {
    return this.previousObject || this.constructor( null )
  }

  tiQuery.fn.find = function( selector ) {
    var returnValue = this.pushStack()

    Array.prototype.push.apply( returnValue, Tizzle.find( selector, tiQuery(this).toArray() ))

    return returnValue
  }

  tiQuery.fn.children = function() {
    var newStack = this.pushStack()

    this.each(function() {
      if ( this.kids )
        Array.prototype.push.apply( newStack, this.kids )
    })

    return newStack
  }

  tiQuery.fn.filter = function( selector ) {
    var newStack = this.pushStack()
    ,   newElements = []

    if ( typeof selector === 'string' ) {
      newElements = Tizzle.filter( selector, this.toArray() )
    } else if ( typeof selector === 'function' ) {
      this.each(function() {
        if ( selector.call( this ))
          newElements.push( this )
      })
    }

    Array.prototype.push.apply( newStack, newElements )

    return newStack
  }

  tiQuery.fn.toArray = function() {
    return Array.prototype.slice.apply( this )
  }

  tiQuery.fn.dump = function( additionalComment ) {
    console.log( (additionalComment || "")  + JSON.stringify( this ) )
    this.each(function() {
      console.log( this )
    })
    return this
  }

  tiQuery.fn.get = function( index ) {
    return this[ index ]
  }

  tiQuery.fn.first = function() {
    return this[0]
  }

  tiQuery.fn.attr = function( attrName, attrValue ) {
    if (typeof attrValue !== 'undefined' )
      return this.setAttribute( attrName, attrValue )
    else if ( typeof attrName === 'string' )
      return this.getAttribute( attrName )
    else return this.setAttributes( attrName )
  }

  tiQuery.fn.getAttribute = function( attrName ) {
    var first = this.first()

    if ( first ) {
      return tiQuery.getters[ attrName ]
        ? tiQuery.getters[ attrName ].call( first )
        : first[ attrName ]
    } else return
  }

  tiQuery.fn.setAttribute = function( attrName, attrValue ) {
    return this.each( function() {
      if ( tiQuery.setters[ attrName ] ) {
        tiQuery.setters[ attrName ].call( this, attrValue )
      } else {
        this[attrName] = attrValue
      }
    })
  }

  tiQuery.fn.setAttributes = function( attributes ) {
    var object = this
    Object.keys( attributes ).forEach( function( attrName ) {
      object.setAttribute( attrName, attributes[ attrName ] )
    })
    return object
  }

  tiQuery.fn.addClass = function( className ) {
    return this.each( function() {
      var $this = tiQuery( this )
      $this.attr( 'classes', $this.attr( 'classes' ).concat( className ))
    })
  }

  ;'click doubletap singletap swipe change complete'.split(' ')
    .forEach( function( eventName ) {
      tiQuery.fn[ eventName ] = function( fn ) {
        return typeof fn === 'function'
          ? this.on( eventName, fn )
          : this.trigger( eventName )
      }
    })

  tiQuery.fn.trigger = function( eventName, event ) {
    return this.each( function() {
      this.fireEvent( eventName, event )
    })
  }

  tiQuery.fn.on = function( eventNames, fn ) {
    eventNames = Array.prototype.slice.call( arguments, 0, -1 )
    fn = arguments[ arguments.length - 1 ]

    return this.each( function() {
      var element = this

      eventNames.forEach( function( eventName ) {
        element.addEventListener( eventName, tiQuery.bind( fn, element ))
      })
    })
  }

  tiQuery.fn.bind = tiQuery.fn.on

  ;'focus blur'.split(' ')
    .forEach( function( eventName ) {
      tiQuery.fn[ eventName ] = function( fn ) {
        return this.each(function() {
          if ( typeof fn === 'function' ) {
            this.addEventListener( eventName, tiQuery.bind( fn, this ))
          } else this[ eventName ]()
        })
      }
    })


  tiQuery.console = console

  tiQuery.config = {
    windowPath: "./",
    jsonPath: "./"
  }

  var addShadowToTop = function( view ) {
    view.add({
      top: -9,
      left: 0,
      width: 320,
      height: 9,
      template: 'top_shadow'
    })
  }

  var coverUpPre = function( from, to ) {
    addShadowToTop( to )
  }

  var revealDownPre = function( from, to ) {
    addShadowToTop( from )
    //to.attr( 'zIndex', -1 )
  }

  tiQuery.transitions = {
    up:    { oldLeft: 0, oldTop: 1, oldZ: -1, left: 0, top: 1, duration: 300 },
    down:  { oldLeft: 0, oldTop: -1, oldZ: -1, left: 0, top: -1, duration: 300 },
    left:  { oldLeft: 1, oldTop: 0, oldZ: -1, left: 1, top: 0, duration: 200 },
    right: { oldLeft: -1, oldTop: 0, oldZ: -1, left: -1, top: 0, duration: 200 },
    coverup:    { oldLeft: 0, oldTop: 0, oldZ: -1, left: 0, top: 1, postLoad: true, duration: 350, pre: coverUpPre },
    revealdown:  { oldLeft: 0,  oldTop: -1, oldZ: 0, left: 0, top: 0, postLoad: true, duration: 350, pre: revealDownPre },
    zoom:  { oldLeft: 0, oldTop: 0, oldZ: -1, left: 0, top: 0, duration: 350, scale: 0.1, opacity: 0.1 },
    zoomout:  { oldLeft: 0, oldTop: 0, oldZ: -1, left: 0, top: 0, postLoad: true, duration: 350, scale: 5, opacity: 0.1 },
    fade:  { oldLeft: 0, oldTop: 0, oldZ: -1, left: 0, top: 0, duration: 100, opacity: 0.1 }
  }


  tiQuery.transition = function( from, to, transitionName, callback ) {
    console.log( 'Transition: ' + transitionName )
    var width = Ti.Platform.displayCaps.platformWidth
    ,   height = Ti.Platform.displayCaps.platformHeight
    ,   transition = tiQuery.transitions[ transitionName ]
    ,   fromAnimationComplete = false
    ,   toAnimationComplete = false

    var finish = function() {
      if ( from ) from.hide() // Otherwise the timing is off

      to.attr( 'touchEnabled', true )
      to.attr( 'zIndex', 0 )
      if ( callback ) callback()
    }

    if ( !transitionName ) {
      to.show()
      finish()
      return
    }

    if (transition.pre) transition.pre( from, to )

    if ( from ) {
      from.attr( 'touchEnabled', false )
      from.attr( 'zIndex', transition.oldZ )
      from.animate({
        left: -( transition.oldLeft * width ),
        top: -( transition.oldTop * height ),
        duration: transition.duration,
        opacity: transition.opacity || 1,
        curve: 'linear'
      }, function() {
        fromAnimationComplete = true
        if ( toAnimationComplete ) finish()
      })
    }

    to.attr({
      left: transition.left * width,
      top: transition.top * height,
      scale: transition.scale || 1,
      opacity: transition.opacity || 1,
      visible: true
    }).animate({
      left: 0,
      top: 0,
      scale: 1,
      opacity: 1,
      curve: 'linear',
      duration: transition.duration
    }, function() {
      toAnimationComplete = true
      if ( fromAnimationComplete ) finish()
    })
    to.show()
  }

  tiQuery.globals = { tiQuery: tiQuery }

  tiQuery.openTiWindowIfNeeded = function() {
    // Only open one window.
    if (!tiQuery.window) {
      tiQuery.window = tiQuery({
        type: 'window',
        background: 'white',
        fullscreen: true,
        width: 320,
        height: 480
      }).first()
      tiQuery.window.open()
    }
  }

  tiQuery.persistentRoots = {}
  tiQuery.rootName = ''

  tiQuery.registerPersistentRoot = function( options ) {
    options = options || {}
    var name = options.name
    ,   root = options.root
    ,   showFunc = options.showFunc
    ,   hideFunc = options.hideFunc
    ,   wait

    if ( typeof options.wait === 'undefined' )
      wait = true
    else
      wait = options.wait

    tiQuery.persistentRoots[ tiQuery.rootName ] = {
      root : tiQuery.root,
      showFunc: showFunc,
      hideFunc: hideFunc,
      wait: wait,
      idCache: {}
    }
  }

  tiQuery.openWindow = function( name, attributes, path, callback ) {
    console.log( '[OPEN WINDOW] ' + name )
    if ( typeof attributes === 'undefined' )
      attributes = {}

    if ( typeof path === 'undefined' )
      path = tiQuery.config.windowPath + name + '.js'

    tiQuery.openTiWindowIfNeeded()

    // The actual work
    var rootView
    ,   rootBeforeView
    ,   rootBeforeName
    ,   transition = attributes['transition']
    ,   wait = false

    if ( typeof attributes.wait !== 'undefined' )
      wait = attributes['wait']

    if ( tiQuery.root ) {
      rootBeforeView = tiQuery.root
      rootBeforeName = tiQuery.rootName
      if ( tiQuery.persistentRoots[ tiQuery.rootName ] ) {
        // This window knows how to persist itself, save off the cache
        tiQuery.persistentRoots[ tiQuery.rootName ].idCache = tiQuery.cache.ids
      }
      tiQuery.cache.clear()
    }

    tiQuery.rootName = name
    if ( tiQuery.persistentRoots[name] ) {
      // Just set the root to the persistent view if it's persistent
      console.log('Recycling root for ' + name)
      rootView = tiQuery.root = tiQuery.persistentRoots[ name ].root
      tiQuery.cache.ids = tiQuery.persistentRoots[name].idCache
      wait = tiQuery.persistentRoots[ name ].wait
    } else {
      // Instantiate a new view if it's not (or doesn't exist yet)
      console.log('Instantiating new root for ' + name)
      rootView = tiQuery.root = tiQuery({
        type: 'view',
        id: name,
        width: 320,
        height: 480,
        touchEnabled: false,
        visible: false,
        background: '#ffffff',
        zIndex: 0
      })
      rootView.attr( attributes )
      tiQuery.window.add(rootView.first())
    }

    var loadRoot = function( name, path ) {
      $.time('[WINDOW LOAD] ' + name, function() {
        if ( tiQuery.persistentRoots[ name ] ) {
          if ( tiQuery.persistentRoots[ name ].showFunc ) {
            // Tell persistent views they're being shown instead of including them
            tiQuery.persistentRoots[ name ].showFunc()
          }
        } else {
          Ti.include( path )
          // FIXME: Here because it seems to help make other event listeners work properly
          //$('*').on('foobar', function() {
          //  console.log('foobar ' + this.id )
          //})
        }
      })
    }

    var cleanupBefore = function() {
      if ( rootBeforeView ) {
        rootBeforeView.trigger( 'close' )
        if ( tiQuery.persistentRoots[ rootBeforeName ] ) {
          // Tell the persistent view that we're going to hide it
          if ( tiQuery.persistentRoots[ rootBeforeName ].hideFunc ) {
            tiQuery.persistentRoots[ rootBeforeName ].hideFunc()
          }
        } else {
          // Non-persistent views get removed entirely.
          tiQuery.window.remove( rootBeforeView.first() )
        }
        rootBeforeView = undefined
      }
    }

    var getRidOfWaitingView = function() {
      if (newViewDone && transitionDone) {
        console.log('Transition away from waiting view')
        tiQuery.transition( tiQuery.waitingView, rootView, transition, function() {
          rootView.trigger( 'transitionfinished' )
        })
        getRidOfView = undefined
      }
    }

    if ( !wait || (transition && tiQuery.transitions[transition].postLoad) ) {
      console.log('no delayed transition')
      // Load new window
      loadRoot( name, path )

      // Transition to new window and clean up old
      tiQuery.transition( rootBeforeView, rootView, transition, function() {
        cleanupBefore()
        rootView.trigger( 'transitionfinished' )
      })
    } else if ( wait ) {
      var newViewDone = transitionDone = false
      // Transition to waiting view
      tiQuery.transition( rootBeforeView, tiQuery.waitingView, transition, function() {
        // Cleanup previous window when done with transition to waiting
        cleanupBefore()
        transitionDone = true
        getRidOfWaitingView()
      })

      loadRoot( name, path )

      newViewDone = true
      // Transition away from waiting view
      getRidOfWaitingView()
    }

  }


  tiQuery.loadJSON = function( name ) {
    var file = Ti.Filesystem.getFile(
      tiQuery.config.jsonPath,
      name + '.json'
    )

    return JSON.parse( file.read() )
  }

  return tiQuery
})()
